/*
   Watch3 ARM9 Code 
   Maarten Pennings - 2007 nov 16
   Based on the devkitPro template
*/

#include "nds.h"
#include <nds/arm9/console.h> //basic print funcionality
#include <stdio.h>

char* Months[] = { "dummy", "jan", "feb", "mar", "apr", "may", "jun", "jul", "aug", "sep", "oct", "nov", "dec" };
char* Weekdays[] = { "sun", "mon", "tue", "wed", "thu", "fri", "sat" };

void DrawKite(float z, float width, float height) 
{ 
  glBegin(GL_QUADS); 
  glVertex3f( -width, 0       , z); 
  glVertex3f( 0     , -width  , z); 
  glVertex3f( +width, 0       , z); 
  glVertex3f( 0     , +height , z); 
  glEnd(); 
} 

void DrawQuad(float z, float width, float height) 
{ 
  glBegin(GL_QUADS); 
  glVertex3f( -width, -height , z); 
  glVertex3f( +width, -height , z); 
  glVertex3f( +width, +height , z); 
  glVertex3f( -width, +height , z); 
  glEnd(); 
} 

void InitGraphics() {
  // Turn on everything 
  powerON(POWER_ALL); 

  //put 3D on top 
  lcdMainOnTop(); 

  // Setup the Main screen for 3D 
  videoSetMode(MODE_0_3D); 

  // IRQ basic setup 
  irqInit(); 
  irqSet(IRQ_VBLANK, 0); 

  // Set our viewport to be the same size as the screen 
  glViewPort(0,0,255,191); 

  // Specify the Clear Color and Depth 
  glClearColor(0,0,0,31); 
  glClearDepth(0x7FFF); 

  consoleDemoInit();
  
  // Reset the screen and setup the view 
  glInit(); 
  
  //ds specific, several attributes can be set here    
  glPolyFmt(POLY_ALPHA(31) | POLY_CULL_NONE); 
  
  // Set the current matrix to be the model matrix 
  glMatrixMode(GL_MODELVIEW); 
  glLoadIdentity(); 
  
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(35, 256.0 / 192.0, 0.1, 100);
  gluLookAt(  0.0, 0.0, 3.5,    //camera possition 
              0.0, 0.0, 0.0,    //look at
              0.0, 1.0, 0.0);   //up
}

int main(void) {
  InitGraphics();
  
  printf("\x1b[5;3HWatch3 - Maarten Pennings\n");

  while(1) {
    int year=    IPC->time.rtc.year+2000;
    int month=   IPC->time.rtc.month;
    int day=     IPC->time.rtc.day;
             
    int weekday= IPC->time.rtc.weekday;
    int hours=   IPC->time.rtc.hours;
    int minutes= IPC->time.rtc.minutes;
    int seconds= IPC->time.rtc.seconds;

    //draw second hand 
    glPushMatrix();    
    glColor3f(0,0,1); 
    glRotateZ(-seconds * 360 / 60); 
    glTranslatef(0,1.8,0); 
    DrawKite(.03,0.15,0.15); 
    glPopMatrix(1); 

    //draw minute hand 
    glPushMatrix();    
    glColor3f(0,1,0); 
    glRotateZ(- (minutes*60 + seconds) / 10); 
    DrawKite(0.02,0.15,1.85); 
    glPopMatrix(1); 

    //draw hourhand 
    glPushMatrix();    
    glColor3f(1,0,0); 
    glRotateZ(- (hours*3600 + minutes*60 + seconds) / 120); 
    DrawKite(0.01,0.20,1.50); 
    glPopMatrix(1);
    
    {
    	int h;
    	for( h=0; h<12; h++ ) {
		    //draw hour tick
		    glPushMatrix();    
		    glColor3f(1,1,1); 
		    glRotateZ(-h * 360 / 12); 
        glTranslatef(0,1.9,0); 
		    DrawQuad(0.00,(h%3==0?0.10:0.03),0.15); 
		    glPopMatrix(1);
    	}
    }

    printf( "\x1b[7;2H" );
    printf( "%i-%s-%i (%s), ", year, Months[month], day, Weekdays[weekday] );
    printf( "%2i:%02i:%02i \n", hours, minutes, seconds );
    
    glFlush(0); 
  }
}
